# Developing Application with DTT Intel Context Sensing Service
## Overview
You will need the class library ContextSensingClient.dll for communicating with the service. 

The entry point to the API is the `ClientBackendLink` class which gives access to Walk Away Lock (WAL), Wake on Approach (WOA), 
No Lock on Presence (NLOP) and Adaptive Dimming features. As feature enable/disable calls are asynchronous 
the callback object given to the class constructor is used to indicate status.
		
## API Reference

First we define the supported features
```csharp
namespace ContextSensingClient
{
	public enum FeatureType
	{
		LOCK = 1,
		WAKE,
		APPLICATION,
		NOLOCKONPRESENCE,
		ADAPTIVEDIMMING
	}
}
```
and support enums for command responses and errors
```csharp
	public enum ResponseStatus
	{
		ENABLE_SUCCESS = 0,
		DISABLE_SUCCESS,
		NO_CHANGE,
		UNKNOWN
	}

	public enum EventType
	{
		EVENT_EXTERNAL_MONITOR = 1,
		EVENT_WAL,
		EVENT_WOA,
		PROXIMITY, // DEPRECATED
		EVENT_SERVICE_RECOVERED,
		EVENT_DTT_STARTED,
		UNKNOWN
	}

	public enum ErrorType
	{
		ServiceVersionIncompatible = 0,
		ServiceStopped,
		ServiceUnavailable,
		ServiceRecoveryInProgress,
		EnableError,
		DisableError,
		GetOptionsError,
		SetOptionError,
		DttUnavailable,
		Success
	}
	
	public class Error
	{
		public ErrorType ErrorType { get; set; }
		public string Description { get; set; }
	}
	
	public enum SettingsAccessType
	{
		LOCAL = 0,
		GLOBAL = 1,
		RESERVED = 2
	}
	
	public enum HumanPresenceStatus
	{
		NOT_PRESENT = 0,
		DISENGAGED = 1,
		ENGAGED = 2,
		FACE_ENGAGED = 3,
		INVALID = 99
	}

	public enum HumanPresenceStatusType
	{
		POLICY, // Combined user presence value based on all inputs and user preferences
		SENSOR, // under evaluation
		PLATFORM // under evaluation
	}
	
	public enum FeatureProperty
	{
		AppName,
		DimmingInterval,
		DimScreen,
		DisengagedDimmingInterval,
		DisengagedDimTarget,
		DisengagedDimWaitTime,
		DisplayOffAfterLock,
		EnabledEvents,
		EnableOnBattery,
		EnableWithExtMonitor,
		ExternalMonitor,
		FailSafeTimeout,
		LaunchOnUserLogIn,
		LockAfterDimInterval,
		LowBatteryRemainingPercentageLimit,
		Misprediction1DimWaitTime,
		Misprediction2DimWaitTime,
		Misprediction3DimWaitTime,
		Misprediction4DimWaitTime,
		MispredictionFaceDetection,
		MispredictionTimeWindow,
		MonitorApp,
		NotPresentDimTarget,
		PowerRequest,
		PreDimInterval,
		PrivacyEnabled,
		Protocol,
		UseLyncSDK,
		WaitInterval,
		WakeOnLowBattery,
		
		/* Deprecated */
		DisplayOff,
		LowBatteryRemainingMinutesLimit,
		MockItemsDir,
		PlaySoundOnLock,
		UserInteraction,
		ValidLidModeList,
		ValidHingeAngleRangeList
	}
 
	public class FeatureSetting
	{
	}

	public class LockFeatures : FeatureSetting
	{
		public bool WALEnabled;
		public int PreDimInterval;
		public int DimmingInterval;
		public int NotPresentDimTarget;
		public int LockAfterDimInterval;
		public int WaitInterval;
		public bool DimScreen;
		public bool ExternalMonitor;
		public bool EnableLockWithExternalMonitor;
		public bool PowerRequest;
		public bool DisplayOffAfterLock;
		public bool UseLyncSDK;

		/* Deprecated */
		public bool Lock;
		public bool DisplayOff;
		public string PlaySoundOnLock;
		public Dictionary<string, List<string>> PowerRequestCategories; 
		public bool UserInteraction;
		public List<int> ValidLidModeList;
		public List<List<int>> ValidHingeAngleRangeList;
	}

	public class WakeFeatures : FeatureSetting
	{
		public bool WOAEnabled;
		public bool WakeOnLowBattery;           //This feature is on when set to be false		
		public bool ExternalMonitor;
		public bool EnableWakeWithExternalMonitor;
		public int LowBatteryRemainingPercentageLimit;

		/* Deprecated */
		public int LowBatteryRemainingMinutesLimit;
		public List<int> ValidLidModeList;
		public List<List<int>> ValidHingeAngleRangeList;
	}

	public class NoLockOnPresenceFeatures : FeatureSetting
	{
		public bool NLoPEnabled;
		public int WaitInterval;
		public bool ExternalMonitor;
		public bool EnableNLoPWithExternalMonitor;
		public bool EnableOnBattery;
		public int LowBatteryRemainingPercentageLimit;
		public int FailSafeTimeout;

		/* Deprecated */
		public int LowBatteryRemainingMinutesLimit;
	}

	public class AppFeatures : FeatureSetting
	{
		public bool MonitorApp;
		public bool LaunchOnUserLogIn;
		public string AppName;
		public string Protocol;
		public List<string> EnabledEvents;
		public bool PrivacyEnabled;
	}
	
	public class AdaptiveDimmingFeatures : FeatureSetting
	{
		public bool ADEnabled;
		public bool ExternalMonitor;
		public bool EnableAdaptiveDimmingWithExternalMonitor;
		public int PreDimInterval;
		public bool MispredictionFaceDetection;
		public int MispredictionTimeWindow;
		public int Misprediction1DimWaitTime;
		public int Misprediction2DimWaitTime;
		public int Misprediction3DimWaitTime;
		public int Misprediction4DimWaitTime;
		public int DisengagedDimmingInterval;
		public int DisengagedDimTarget;
		public int DisengagedDimWaitTime;
	}

```
Now we define callback required for enabling/disabling features
```csharp
namespace ContextSensingClient
{
	public interface IFeatureCallback
	{
		// Called when feature has been successfully enabled or disabled
		void OnSuccess(FeatureType featureType, ResponseType responseType);
		// Called on an event (e.g. on a WAL proximity distance change)
		void OnEvent(FeatureType featureType, EventType eventType, object eventPayload);
		// Called on error
		void OnError(FeatureType featureType, Error state);
	}
}
```
Now the `ClientBackendLink` class definition
```csharp
namespace ContextSensingClient
{
	class ClientBackendLink
	{
		ClientBackendLink(IFeatureCallback callback);     // Constructor takes callabck object
		void Enable(FeatureType featureType);             // Enables specified feature. Result is handled by callback
		void Disable(FeatureType featureType);            // Disables specified feature. Result is handled by callback
		Task SetOption(FeatureType featureType, 		// Returns a task that that sets a feature property using specified key and value
						FeatureProperty property, 
						object propertyValue, 
						SettingsAccessType settingsMode = SettingsAccessType.LOCAL);                                    
		Task<FeatureSetting> GetOptions(				// Returns a task that is used to make GetOptions call for specified feature.
						FeatureType featureType, 
						SettingsAccessType settingsMode = SettingsAccessType.LOCAL);
		Task<HumanPresenceStatus> GetHumanPresenceStatus(HumanPresenceStatusType presenceType); // Gets current presence status for specified type
	}
}
```
## Sample console application
This sample shows how to enable/disable some of the features and get options
```csharp
using System;
using System.Threading;
using System.Diagnostics;
using ContextSensingClient;
using System.Threading.Tasks;

namespace HupConsoleApp
{
	class HupListener : IFeatureCallback
	{
		private bool error;
		private ManualResetEvent responseEvent;
		private Stopwatch stopWatch;

		public HupListener()
		{
			responseEvent = new ManualResetEvent(false);
			Reset();
			stopWatch = new Stopwatch();
			stopWatch.Start();
		}

		public void Reset()
		{
			error = false;
			responseEvent.Reset();
		}

		public void OnSuccess(FeatureType featureType, ResponseType responseType)
		{
			Console.WriteLine("{0} {1}", featureType, responseType);
			responseEvent.Set();
		}

		public void OnEvent(FeatureType featureType, EventType eventType, object eventPayload)
		{
			Console.Write("{0}: {1} OnEvent: {2}", stopWatch.ElapsedMilliseconds, featureType, eventType.ToString());
			if (eventType == EventType.EVENT_WAL)
			{
				Console.Write("Received walk away lock event!");
			}
			Console.WriteLine();
		}

		public void OnError(FeatureType featureType, Error state)
		{
			error = true;
			Console.WriteLine("{0} OnError: {1}", featureType, state.Description);
			responseEvent.Set();
		}

		public int Wait(int timeoutMs)
		{
			var stopwatch = new Stopwatch();
			stopwatch.Start();
			if (responseEvent.WaitOne(timeoutMs))
				return (int)stopwatch.ElapsedMilliseconds;
			else
				return -1;
		}

		public void WaitLog(int timeoutMs)
		{
			int responseTime = Wait(timeoutMs);
			if (responseTime < 0)
			{
				Console.WriteLine("Error - no response");
				return;
			}
			if (!error)
			{
				Console.WriteLine("Succeeded in {0}ms", responseTime);
			}
			else
			{
				Console.WriteLine("Failed");
			}
		}
	}

	class Program
	{
		static void Main(string[] args)
		{
			bool finished = false;
			HupListener hupListener = new HupListener();
			ClientBackendLink client = new ClientBackendLink(hupListener);

			Console.WriteLine("Select command");
			Console.WriteLine(" 1 - GetOptions(WAL)");
			Console.WriteLine(" 2 - GetOptions(WOA)");
			Console.WriteLine(" 0 - Exit");
 
			do
			{
				ConsoleKeyInfo option = Console.ReadKey(true);
				switch (option.KeyChar)
				{
					case '0':
						finished = true;
						break;
					case '1':
						Console.WriteLine("Calling GetOptions for WAL");
						hupListener.Reset();
						Task<FeatureSetting> lockTask = client.GetOptions(FeatureType.LOCK);
						lockTask.Wait();
						Console.WriteLine(lockTask.Result);
						break;
					case '2':
						Console.WriteLine("Calling GetOptions for WOA");
						hupListener.Reset();
						Task<FeatureSetting> wakeTask = client.GetOptions(FeatureType.WAKE);
						wakeTask.Wait();
						Console.WriteLine(wakeTask.Result);
						break;
				}
			} while (!finished);
		}
	}
}
```
## Handling Errors
* If service fails, ClientBackendLink callback will receive an OnError() call with code set to `ServiceRecoveryInProgress`
* OS will then try to re-start service with three possible outcomes
  * Up to 2 futher OnError() calls with error code set to `ServiceRecoveryInProgress` indiciating OS is still trying to recover service
  * An OnEvent() call with event set to `EVENT_SERVICE_RECOVERED` indicating service is back up
  * An OnError() call with error code set to `ServiceUnavailable` indiciating OS was unable to recover service and it should be considered stopped and unusable.
  
* If DTT stops or is otherwise not available, an OnError() call with error code set to 'DttUnavailable' will be received. 
* If DTT is unavailable and then becomes available (e.g. Remote Desktop session end), an OnEvent() call with event set to 'EVENT_DTT_STARTED' will be received.

## Notes on Using GetOptions()
The `GetOptions` method ensures that the client is aware of service settings, so should be called in the following situations
* When starting application
* When recovering from service errors
* When starting the service with a trigger start
* On a user session switch
If a `SetOption()` call fails, the U/I must be aware that the new value has not taken. A brute force solution is to make a GetOptions call and update U/I with result.
## Using Service Triggers
If the service is configured to use [trigger start with custom events](https://docs.microsoft.com/en-us/windows/win32/services/service-trigger-events). 
The advantage of service triggers is that you do not need admin privildges to issue a start requests. Event details are as follows
* Event UUID is F8445F8F-18D5-4076-A2C0-9D1669782993
* To start service send event with string 'Start'

The sample application includes a class `ServiceTrigger` that can detect if triggering is enabled, start and stop the service. 
The start and stop methods both return 0 if event was successfully sent, otherwise a Windows error code. Checking that service is running is the responsibility of the caller.
The is no .NET support for trigger events, so P/Invoke and unmanaged data marshalling must be used. 

```csharp
class ServiceTrigger
{
	bool IsEnabled();
	uint Start();
}
```

## Notes on Using GetHumanPresenceStatus()
GetHumanPresenceStatus is used to query the current presence value that ICSS/DTT has evaluated.
* SENSOR - Raw sensor data 
* PLATFORM - SENSOR + HID activity
* POLICY - Combined final presence value that includes PLATFORM and user preferences (e.g user doesn't care about external monitor, so it won't be factored in)
SENSOR and PLATFORM are under evaluation and are not recommended for production use at this time.

## Notes on Opting In/Out of Vision Sensor usage
User selection on using the vision sensor (if available on the system) is presented as the 'PrivacyEnabled' application choice. 
When privacy is enabled, registration to the vision sensor will be canceled, and if available the non-vision presence sensor will be used instead.
When a sensor change is successful, an OnSuccess() event will be received with the contextual result of the action; otherwise an error will be surfaced instead.
